% Allow response to be dynamic
	:-dynamic(response/2).


% what colours are we allowed in the sequence?
	colour(blue).
	colour(red).


% what numbers are we allowed in the sequence?
	what(1).
	what(2).
	what(3).
	what(4).
	what(5).


% what is each entry in the sequence made up from?
	move(X-Y) :- what(X), colour(Y).


% systematically generate all possible guesses
	generate_move([A,B,C,D,E]) :- 	move(A),
					move(B),
					move(C),
					move(D),
					move(E).

% calculate left, right, and colour values from a code-item and a guess-item
% the colour value is incremented when a colour is wrong.
	value(CH-CHcol, GH-GHcol, L, R, C) :-	((CH<GH -> L = 1);L=0),
						((CH>GH -> R = 1);R=0),
						((CHcol\=GHcol -> C=1);C=0).


% calculate left, right, and colour values from
% a complete code and a complete guess (base-case)
	val([CH], [GH], L, R, C) :- value(CH, GH, L, R, C).


% calculate left, right, and colour values from
% a complete code and a complete guess (resursive-case)
	val([CH|CT],
	    [GH|GT],
	    L,R,C) :-	val(CT, GT, L2, R2, C2),
			value(CH, GH, L1, R1, C1),
			L is L1+L2,
			R is R1+R2,
			C is C1+C2.


% play the game for a given code.
% First remove all items from the database.
% Generate a move, then filter it to see if it is a worthy guess.
% If it is then ask if it is the code.
% If it is the code then the game is won.
% If it is'nt the code then put the guess in the database and fila, ie try another move.

	play(Code) :-	retractall(response(_,_)),	% empty the response database
			generate_move(M),		% generate a move
			filter(M),			% Is it worthy?
			val(M,Code,L,R,C),		% It is, so get its values (L,R, and C)
			(L==0, R==0, C==0 ->				% Is it right?
				(					% YES
					write('congratulations'),	% game completed
					nl,
					write('The code was: '),
						display(M),
					nl
				);
				(					% NO
					assert(response(M,L-R-C)),	% stick guess with L-R and C in database
					write('guessing '),
					display(M),
					nl,
					fail				% fail this attempt
				)					% and try another guess
			).



% The filter - For the given guess:
% check it against all entries in the database
% as if the entries in the database were the real code.
% If for all database entries the values L-R and C matche then it is a valid guess,
% otherwise its not

	filter(M) :- forall(response(S,L-R-C), val(S,M,L,R,C)).




